local shoutcolor = SKINCOLOR_NONE
local shoutplayer
local shouttype = ""
local shoutxpos = 0
local shoutypos = 0

COM_AddCommand("shout", function(p, arg)
	if gamestate != 1 then
		CONS_Printf(p, "You can only do this in gameplay.")
		return
	end
	
	if p.shoutcooldown or (shoutprogress <= shoutcooldown and shoutprogress > 0) then
		CONS_Printf(p, "You can't shout when on cooldown!")
		return
	end
	
	if shoutallowed == "off" or (shoutallowed == "testimony" and not testimonyprogress) then
		CONS_Printf(p, "You can't shout right now.")
		return
	end
	
	if not arg then
		CONS_Printf(p, "\x82shout <type>\x80 - Make a shout bubble appear on screen.")
		CONS_Printf(p, "\x82".."Available shout types\x80:")
		CONS_Printf(p, "\x82holdit\x80 - \"Hold it!\"")
		CONS_Printf(p, "\x82object\x80 - \"Objection!\"")
		CONS_Printf(p, "\x82takethat\x80 - \"Take that!\"")
		return
	elseif arg != "holdit" and arg != "object" and arg != "takethat" then
		CONS_Printf(p, "Invalid shout type.")
		return
	end
	
	shoutplayer = p
	if recolorcutins then
		shoutcolor = shoutplayer.mo.color
	else
		shoutcolor = SKINCOLOR_RED
	end
	shouttype = arg
	if shoutwarn then
		shoutplayer.consecshouts = $+1
	end
	shoutprogress = 1
	for pl in players.iterate
		P_FlashPal(pl, PAL_WHITE, TICRATE/4)
	end
	
	if arg == "holdit" then
		S_StartSound(nil, p.holdit)
	elseif arg == "object" then
		S_StartSound(nil, p.objection)
	elseif arg == "takethat" then
		S_StartSound(nil, p.takethat)
	end
end)

COM_AddCommand("shoutwarn", function(p, arg)
	if not arg then
		CONS_Printf(p, "\x82shoutwarn\x80 - Punish the player for consecutive shouts. Boolean.")
	elseif not (arg == "off" or arg == "on" or arg == "yes" or arg == "no" or arg == "y" or arg == "n" or arg == "1" or arg == "0" or arg == "true" or arg == "false") then
		CONS_Printf(p, "Invalid argument.")
	end
	
	if (arg == "off" or arg == "no" or arg == "n" or arg == "0" or arg == "false") then
		shoutwarn = false
		CONS_Printf(p, "Disabled consec. shout warning.")
	elseif (arg == "on" or arg == "yes" or arg == "y" or arg == "1" or arg == "true") then
		shoutwarn = true
		CONS_Printf(p, "Enabled consec. shout warning.")
	end
end, COM_ADMIN)

COM_AddCommand("shoutallowed", function(p, arg)
	if not arg then
		CONS_Printf(p, "\x82shoutallowed\x80 - Allow or disallow shouting. \"testimony\" makes the script only allow it mid-testimony.")
	elseif not (arg == "off" or arg == "on" or arg == "testimony" or arg == "yes" or arg == "no" or arg == "y" or arg == "n" or arg == "2" or arg == "1" or arg == "0" or arg == "true" or arg == "false") then
		CONS_Printf(p, "Invalid argument.")
	end
	
	if (arg == "off" or arg == "no" or arg == "n" or arg == "0" or arg == "false") then
		shoutallowed = "off"
		CONS_Printf(p, "Disabled shouts.")
	elseif (arg == "on" or arg == "yes" or arg == "y" or arg == "1" or arg == "true") then
		shoutallowed = "on"
		CONS_Printf(p, "Enabled shouts.")
	elseif (arg == "testimony" or arg == "2") then
		shoutallowed = "testimony"
		CONS_Printf(p, "Enabled shouts, but only during testimonies.")
	end
end, COM_ADMIN)

addHook("MapLoad", do
	shoutprogress = 0
end)

addHook("PlayerSpawn", function(p)
	p.consecshouts = 0
	p.shoutcooldown = 0
end)

addHook("PlayerThink", function(player)
	if not player.consecshouts then player.consecshouts = 0 end
	if not player.shoutcooldown then player.shoutcooldown = 0 end
	
	if not (player.cmd.buttons & BT_CUSTOM1) then
		player.c1r = true
		player.c1d = false
	elseif player.c1r then
		player.c1d = true
		player.c1r = false
	else
		player.c1d = false
	end
	if not (player.cmd.buttons & BT_CUSTOM2) then
		player.c2r = true
		player.c2d = false
	elseif player.c2r then
		player.c2d = true
		player.c2r = false
	else
		player.c2d = false
	end
	if not (player.cmd.buttons & BT_CUSTOM3) then
		player.c3r = true
		player.c3d = false
	elseif player.c3r then
		player.c3d = true
		player.c3r = false
	else
		player.c3d = false
	end
	
	if player.c1d then
		COM_BufInsertText(player, "shout holdit")
	elseif player.c2d then
		COM_BufInsertText(player, "shout object")
	elseif player.c3d then
		COM_BufInsertText(player, "shout takethat")
	end
	
	if player.consecshouts >= 15 then
		player.consecshouts = 0
		player.shoutcooldown = 60*TICRATE
	end
	
	if shoutprogress == 0 and player.consecshouts then
		player.consecshouts = 0
	end
	
	if player.shoutcooldown then
		player.shoutcooldown = $-1
	end
	
	if skinToShout[player.mo.skin] then
		player.holdit = skinToShout[player.mo.skin]["holdit"]
		player.objection = skinToShout[player.mo.skin]["object"]
		player.takethat = skinToShout[player.mo.skin]["tkthat"]
	else
		player.holdit = skinToShout["metalsonic"]["holdit"]
		player.objection = skinToShout["metalsonic"]["object"]
		player.takethat = skinToShout["metalsonic"]["tkthat"]
	end
end)

hud.add(function(v, p, c)
	if not shoutprogress then return end
	
	if shoutprogress <= 17 then
		local randomang = AngleFixed(v.RandomRange(0, ANGLE_MAX))
		shoutxpos = (sin(randomang)*50)/FRACUNIT
		shoutypos = cos(randomang)/FRACUNIT
	else
		shoutxpos = 0
		shoutypos = 0
	end
	
	if shouttype == "holdit" then
		v.draw(160+shoutxpos, 100+shoutypos, v.cachePatch("AAHOLDIT"), 0, v.getColormap("sonic", shoutcolor))
	elseif shouttype == "object" then
		v.draw(160+shoutxpos, 100+shoutypos, v.cachePatch("AAOBJECT"), 0, v.getColormap("sonic", shoutcolor))
	elseif shouttype == "takethat" then
		v.draw(160+shoutxpos, 100+shoutypos, v.cachePatch("AATKTHAT"), 0, v.getColormap("sonic", shoutcolor))
	end
	
	if shoutprogress <= TICRATE then
		shoutprogress = $+1
	else
		shoutprogress = 0
	end
end, "game")

hud.add(function(v, p, c)
	if p.shoutcooldown then return end
	if shoutprogress <= shoutcooldown and shoutprogress > 0 then
		v.drawString(160, 10, "Time until cooldown ends: "..(shoutcooldown-shoutprogress)/TICRATE.."."..((shoutcooldown-shoutprogress)*10/TICRATE)-(((shoutcooldown-shoutprogress)/TICRATE)*10).." seconds", V_ALLOWLOWERCASE|V_YELLOWMAP, "thin-center")
	end
end)

hud.add(function(v, p, c)
	if not shoutwarn then return end
	
	if p.shoutcooldown then
		v.drawString(160, 10, "Time until cooldown ends: "..p.shoutcooldown/TICRATE.."."..(p.shoutcooldown*10/TICRATE)-((p.shoutcooldown/TICRATE)*10).." seconds", V_ALLOWLOWERCASE|V_YELLOWMAP, "thin-center")
		v.drawString(160, 100, "AIGHT, YOU ASKED FOR IT.", V_REDMAP, "center")
	elseif p.consecshouts and p.consecshouts >= 10 then
		v.drawString(160, 100, "YOU'RE IRRITATING ME WITH THE SHOUTING!", V_ORANGEMAP, "center")
	elseif p.consecshouts and p.consecshouts >= 5 then
		v.drawString(160, 100, "CAN YOU PLEASE STOP?", V_YELLOWMAP, "center")
	end
end, "game")